import java.util.NoSuchElementException;
/*
this class implements a fairly standard int queue; it's mainly meant for showing how to test.
it's a linked list implementation, maintaining a front and back.
things get enqueued at the back and dequeued from the front.
the 'next' pointer of each node points backward (that is, toward 'back')
*/

public class AwesomeQueue {
	// enqueue at back, dequeue from front
	private LLNode front, back; //pointers point to back of list

	//initialize front and back to null
	public AwesomeQueue() {
		front = null;
		back = null;
	}

	public boolean isEmpty() {
		return front == null;
	}

	//add to back
	public void enqueue(int v) {
		if (isEmpty()) {
			back = front = new LLNode(v);
		} else {
			back.next = new LLNode(v);
			back=back.next;
		}
	}

	//remove from front
	//throw an exception if its empty
	public int dequeue() {
		if (isEmpty())
			throw new NoSuchElementException();
		int value = front.value;
		if (front == back) // only 1 node; set pointers to null
		{
			front = back = null;
		} else {
			front=front.next;
		}
		return value;
	}
	
	public SimpleIterator<Integer> getIterator()
	{
		return new AwesomeQueueIterator();
	}

	//our linked list node; just has next and value fields
	//it is static because it doesn't need to access its outer class's values
	private static class LLNode {
		private LLNode next;
		private int value;

		public LLNode(int v) {
			value = v;
			next=null;
		}
	}
	
	//this is not static because this class needs to access 'front' from AwesomeQueue
	//note: this assumes that the queue's contents will not be changed after the iterator is created; if it is, strange things may happen
	private class AwesomeQueueIterator implements SimpleIterator<Integer>
	{
		private AwesomeQueue.LLNode node; //you can actually leave off the 'AwesomeQueue.' part, but I find it clearer this way
		private AwesomeQueueIterator()
		{
			node=front; //inner classes have access to the outer class's values (unless the inner class is static)
		}
		
		public Integer next()
		{
			if(!hasNext()) throw new java.util.NoSuchElementException();
			int returnMe=node.value;
			node=node.next;
			return returnMe;
		}
		
		public boolean hasNext()
		{
			return node!=null;
		}
	}
}
